/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "appinterface.h"
#include <QFont>
#include <QDBusReply>
#include <QDir>


 AppInterface ::AppInterface (QString name, QWidget *parent)
{
	setParent ( parent);
	Name = name;

    	LineEdit = new QLineEdit( this );
	connect(LineEdit,SIGNAL(textChanged ( QString )), this, SLOT(LineEditChanged( QString )));

	idlestates << "idle" << "idle_dim" << "idle_prelock" << "lock" << "suspend" << "awake";

	ComboBox = new QComboBox(this);
	ComboBox->addItems(idlestates);


}

void AppInterface ::resizeEvent( QResizeEvent *event)
{
	int wx = event->size().width()/2;
	int wy = event->size().height();
	
	QFont font;
	float fsize = float(wy*20/100);
	font = LineEdit->font();
	font.setPointSize(fsize);

    	LineEdit->setGeometry(0,0,wx,wy);
	LineEdit->setFont(font);

    	ComboBox->setGeometry(wx,0,wx,wy);
	ComboBox->setFont(font);


}



/*!
    \fn Dialog::getPreventState()
 */
QString AppInterface ::getPreventState()
{
	QString state;
	state = "none";

	if (LineEdit->text().length() > 0) {
		
		QDir procdir( "/proc" );
		QStringList entries = procdir.entryList( QDir::AllDirs );

		int i;
		bool isProcess;
		QFile ProcessStatus;
		char buf[100];
		QString nameline;
		i = entries.size()-1; 

		do {
			entries.at(i).toInt( &isProcess, 10);
			if (isProcess) {
				ProcessStatus.setFileName("/proc/" + entries.at(i) + "/status");
				ProcessStatus.open(QIODevice::ReadOnly);
				ProcessStatus.readLine(buf, sizeof(buf));
				nameline.sprintf("%s",buf);
				ProcessStatus.close();
			}
			i=i-1;
		} while ( (i > 0) &&  (!nameline.contains(LineEdit->text(),Qt::CaseSensitive)));

	qDebug() << i;

	
	if ( i > 0 ) state = ComboBox->currentText ();
	
	}


	return state;

}


/*!
    \fn Dialog::CheckBoxChanged()
 */
void AppInterface ::LineEditChanged( QString appname )
{
    	if (appname.isEmpty()) ComboBox->setDisabled(true);
	else ComboBox->setDisabled(false);

}


/*!
    \fn Dialog::~Dialog()
 */
 AppInterface ::~AppInterface ()
{
    /// @todo implement me
}





/*!
    \fn Dialog::saveSettings(QSettings *settings)
 */
void AppInterface ::saveSettings(QSettings *settings)
{
    	settings->beginGroup(Name);
	settings->setValue("name",LineEdit->text ());
	settings->setValue("state",ComboBox->currentText ());
	settings->endGroup();

}


void AppInterface ::loadSettings( QSettings *settings)
{
	QString state,appname;
	int n;

   	settings->beginGroup(Name);
	appname = settings->value("name").toString();
	state = settings->value("state").toString();
	settings->endGroup();

	LineEdit->setText(appname);
	if (appname.isEmpty()) ComboBox->setDisabled(true);

	n = ComboBox->findText( state,Qt::MatchExactly | Qt::MatchCaseSensitive );
	ComboBox->setCurrentIndex(n);

}
