/***************************************************************************
 *   Copyright (C) 2009 by Rico Rommel   *
 *   rico@bierrommel.de   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include "idlehandler.h"
#include <QMessageBox>
#include <QIcon>
#include <QPushButton>
#include <QWidget>
#include <QFont>
#include "devinterface.h"
#include "qcheckbutton.h"

IdleHandler::IdleHandler()
{
	QIcon icon("/usr/share/idlehandler/idlehandler.png");
	setWindowIcon(icon); 
	systrayicon = new QSystemTrayIcon(icon, this);

	connect(systrayicon, SIGNAL(activated(QSystemTrayIcon::ActivationReason)),this, SLOT(iconActivated(QSystemTrayIcon::ActivationReason)));
	systrayicon->show();

	QuitButton = new QPushButton(QObject::tr("Quit"),this);
	connect(QuitButton,SIGNAL(clicked()), qApp, SLOT(quit()));

	CloseButton = new QPushButton(QObject::tr("Close"),this);
	connect(CloseButton,SIGNAL(clicked()), this, SLOT(close()));

	SaveButton = new QPushButton(QObject::tr("Save"),this);
	connect(SaveButton,SIGNAL(clicked()), this, SLOT(saveSettings()));
	do {
		idleinterface = new QDBusInterface("org.freesmartphone.odeviced", "/org/freesmartphone/Device/IdleNotifier/0", "org.freesmartphone.Device.IdleNotifier", QDBusConnection::systemBus(), this);
		if (!idleinterface->isValid ()) sleep(5);
		}
	while (!idleinterface->isValid ());

	connect(idleinterface,SIGNAL( State(QString) ), this, SLOT(IdleStateChanged(QString)));

	do {
		usageinterface = new QDBusInterface("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage", QDBusConnection::systemBus(), this);
		if (!usageinterface->isValid ()) sleep(5);
		}
	while (!usageinterface->isValid ());


	idlestates << "busy" << "idle" << "idle_dim" << "idle_prelock" << "lock" << "suspend" << "awake";


    	settings = new QSettings("openmoko","idlehandler");
	settings->sync ();

	createUSBInterface();
	createGPSInterface();
	createCALLInterface();
	createAppInterface();


}


IdleHandler::~IdleHandler()
{
//    QDBusConnection::disconnectFromBus("dbus");
}


void IdleHandler::iconActivated(QSystemTrayIcon::ActivationReason)
{
    showMaximized ();
}


void IdleHandler::closeEvent ( QCloseEvent * event )
{
     	hide();
        event->ignore();
}





void IdleHandler::resizeEvent( QResizeEvent *event)
{
	int wx = event->size().width();
	int wy = event->size().height();
	
	int space = wx * SPACE / 100;
	int w = wx * WIDTH / 100;
	int h = wy * HEIGHT / 100;

	QFont font;
	float fsize = float(h*40/100);
	font = CloseButton->font();
	font.setPointSize(fsize);

    	CloseButton->setGeometry(space,wy-h-space,w,h);
	CloseButton->setFont(font);
	QuitButton->setGeometry(space+w,wy-h-space,w,h);
	QuitButton->setFont(font);
	SaveButton->setGeometry(space,wy-2*h-space,2*w,h);
	SaveButton->setFont(font);

	USB->setGeometry(space,space,2*w,h);
	GPS->setGeometry(space,2*space+h,2*w,h);
	CALL->setGeometry(space,3*space+2*h,2*w,h);
	APP->setGeometry(space,4*space+3*h,2*w,h);
	

}


void IdleHandler::IdleStateChanged(QString state)
{
	qDebug() << "Empfange Signal " << state; 

	bool goback = false;
	if (state == USB->getPreventState()) goback = true ;
	if (state == GPS->getPreventState()) goback = true ;
	if (state == CALL->getPreventState()) goback = true ;
	if (state == APP->getPreventState()) goback = true ;

	if (goback) {
		int n = idlestates.indexOf( state, 0);
		idleinterface->call("SetState",idlestates.at(n-1));
		qDebug() << "Sending Signal " << idlestates.at(n-1); 
		
	
		}
	else {
		if (state == "suspend") usageinterface->call("Suspend");
		qDebug() << "Sending Signal " << state;
		
		}
		

}


void IdleHandler::createUSBInterface()
{
    	USB = new DeviceInterface(QObject::tr("USB"), this);

	QStringList match; 
	match << "full" << "charging";

	USB->setDBusArguments("org.freesmartphone.odeviced", "/org/freesmartphone/Device/PowerSupply/battery", "org.freesmartphone.Device.PowerSupply","GetPowerStatus","", match);
	USB->loadSettings( settings );

}


void IdleHandler::createGPSInterface()
{
	GPS = new DeviceInterface(QObject::tr("GPS"), this);

	QStringList match; 
	match << "true";

	GPS->setDBusArguments("org.freesmartphone.ousaged", "/org/freesmartphone/Usage", "org.freesmartphone.Usage","GetResourceState","GPS", match);
	GPS->loadSettings( settings );

}



void IdleHandler::createCALLInterface()
{
    	CALL = new DeviceInterface(QObject::tr("Call"), this);

	QStringList match; 
	match << "gsmhandset";

	CALL->setDBusArguments("org.freesmartphone.odeviced", "/org/freesmartphone/Device/Audio", "org.freesmartphone.Device.Audio","GetScenario","", match);

	CALL->loadSettings( settings );

}


/*!
    \fn IdleHandler::saveSettings()
 */
void IdleHandler::saveSettings()
{
	USB->saveSettings( settings );
	GPS->saveSettings( settings );
	CALL->saveSettings( settings );
	APP->saveSettings( settings );

	settings->sync ();   
}



void IdleHandler::createAppInterface()
{
       	APP = new AppInterface(QObject::tr("Application"), this);
	APP->loadSettings( settings );

}
